﻿//Collidable.cs
//This is the parent class from which all objects which must perform
//collision detection shall inherit.
//Author: Weston Castleberg

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SuperScooter
{
    /// <summary>
    /// This enum defines the types of Collidable objects which individual instances
    /// of the Collidable class can be.  A passable object may be crossed by another
    /// passable object.  An impassable object may not be passed.  A platform object
    /// may be passed from the bottom or from the sides, but may not be passed from 
    /// the top.  A boundary object may or may not be passed depending on the behavior
    /// of the object attempting to pass it
    /// </summary>
    public enum CollideType { Passable, Impassable, Platform, Boundary, Enemy2Platform, Collectable };

    public class Collideable
    {
        //the bounding box around the object
        protected Rectangle shell;
        //the picture which represents the object
        protected Texture2D avatar;
        //the type of Collidable object which the object represents
        CollideType pass;


        /// <summary>
        /// Default Constructor: initializes the bounding rectangle to (0,0) position
        /// with a height and width of 0.  Sets avatar to NULL, and sets pass to the
        /// default of passable
        /// </summary>
        public Collideable()
        {
            shell = new Rectangle(0, 0, 0, 0);
            avatar = null;
            pass = CollideType.Impassable;

        }

        /// <summary>
        /// Returns the lowest y-coordinate (upper bound) of the object's 
        /// bounding rectangle.
        /// </summary>
        /// <returns>int</returns>
        public virtual int UpperBound
        {
            get
            {
                return shell.Top;
            }

        }

        /// <summary>
        /// Returns the highest x-coordinate (right bound) of the object's
        /// bounding rectangle.
        /// </summary>
        /// <returns>int</returns>
        public virtual int RightBound
        {
            get
            {
                return shell.Right;
            }

        }

        /// <summary>
        /// Returns the highest y-coordinate (lower bound) of the object's
        /// bounding rectangle.
        /// </summary>
        /// <returns>int</returns>
        public virtual int LowerBound
        {
            get
            {
                return shell.Bottom;
            }

        }

        /// <summary>
        /// Returns the lowest x-coordinate (left bound) of the object's
        /// bounding rectangle.
        /// </summary>
        /// <returns>int</returns>
        public virtual int LeftBound
        {
            get
            {
                return shell.Left;
            }

        }

        /// <summary>
        /// Returns the width of the private data member avatar.
        /// </summary>
        /// <returns>int</returns>
        public virtual int Width
        {
            get
            {
                return avatar.Width;
            }

        }

        /// <summary>
        /// Returns the height of the private data member avatar.
        /// </summary>
        /// <returns>int</returns>
        public virtual int Height
        {
            get
            {
                return avatar.Height;
            }

        }

        /// <summary>
        /// Returns a copy of the object's bounding rectangle.
        /// </summary>
        /// <returns>Rectangle</returns>
        public virtual Rectangle BoundingRectangle
        {
            get
            {
                return shell;
            }

        }

        /// <summary>
        /// This is a property which will get or set the private data member
        /// avatar
        /// </summary>
        public virtual Texture2D Avatar
        {
            get
            {
                return avatar;
            }
            set
            {
                avatar = value;
            }

        }

        /// <summary>
        /// This property gets or sets the value of the private data member
        /// pass.  This variable determines the type of Collidable object
        /// which this is.
        /// </summary>
        public virtual CollideType Pass
        {
            get
            {
                return pass;
            }
            set
            {
                pass = value;
            }

        }

        /// <summary>
        /// Sets the position of the object's bounding rectangle, and thus its position
        /// on the screen, by using the vector passed in as the top-left corner and 
        /// then creating the bounds of the box based upon the height and width of the 
        /// avatar
        /// </summary>
        /// <param name="pos">the top left corner of the object's bounding rectangle</param>
        public virtual void SetPosition(Vector2 pos)
        {
            if (avatar == null)
                shell = new Rectangle((int)pos.X, (int)pos.Y, 40, 40);

            else
                shell = new Rectangle((int)pos.X, (int)pos.Y, avatar.Width, avatar.Height);

        }

        /// <summary>
        /// Changes the position of the object's bounding rectangle, and thus its position
        /// on the screen, by using the vector passed in as the change in x and y coordinates
        /// </summary>
        /// <param name="change">The change in x and y coordinates</param>
        public virtual void ChangePosition(Vector2 change)
        {
            if (avatar == null)
                shell = new Rectangle(LeftBound + (int)change.X, UpperBound + (int)change.Y, 40, 40);

            else
                shell = new Rectangle(LeftBound + (int)change.X, UpperBound + (int)change.Y, Width, Height);
        }

        /// <summary>
        /// This returns true if the bounding rectangle of the object passed in has collided
        /// with the bounding rectangle of the object pointed to by this.  Note that no
        /// collision takes place if one or both of the objects is passable.
        /// </summary>
        /// <param name="c">The Collidable object with which this object might have collided</param>
        /// <returns>true/false collision took place</returns>
        public virtual bool Collision(Collideable c)
        {
            if (pass == CollideType.Passable && c.pass == CollideType.Passable)
                return false;

            //if the two shells are intersecting, or if they are merely touching, then there
            //is a collision
            return shell.Intersects(c.BoundingRectangle) || (UpperBound - 1 == c.LowerBound) ||
                (RightBound + 1 == c.LeftBound) || (LowerBound + 1 == c.UpperBound) ||
                (LeftBound - 1 == c.RightBound);

        }
 
        public virtual void Update()
        {
        }


    }
}
